﻿using OpenTK.Graphics.OpenGL;

namespace Engine
{
    public class VertexBuffer
    {
        // Private constructor because it is Singleton
        private VertexBuffer() { }
        private static VertexBuffer _instance;
        public static VertexBuffer GetInstance()
        {
            if (_instance == null)
            {
                _instance = new VertexBuffer();
            }
            return _instance;
        }

        private int _vertexBuffer;
        private int _textureBuffer;

        public int GetVertexBuffer()
        {
            return _vertexBuffer;
        }

        public int GetTextureBuffer()
        {
            return _textureBuffer;
        }

        public bool Initialize()
        {
            // Create a buffer for our vertex positions
            GL.GenBuffers(1, out _vertexBuffer);

            // Activate vertexBuffer
            GL.BindBuffer(BufferTarget.ArrayBuffer, _vertexBuffer);

            float[] vertices = new float[]
            {
                //-0.5f, -0.5f,
                //0.5f, -0.5f,
                //-0.5f, 0.5f,
                //0.5f, 0.5f
                0f, 0f,
                1f, 0f,
                0f, 1f,
                1f, 1f
            };

            // Write the data into the vertexBuffer
            GL.BufferData(BufferTarget.ArrayBuffer, vertices.Length * sizeof(float), vertices, BufferUsageHint.StaticDraw);

            // Create a buffer for texture coordinates
            GL.GenBuffers(1, out _textureBuffer);

            // Activate textureBuffer
            GL.BindBuffer(BufferTarget.ArrayBuffer, _textureBuffer);

            // Define the corresponding texture cooridnates
            float[] textureCoords = new float[]
            {
                0f, 0f,
                1f, 0f,
                0f, 1f,
                1f, 1f
            };

            // Loads textureCoords into the vertexBuffer
            GL.BufferData(BufferTarget.ArrayBuffer, textureCoords.Length * sizeof(float), textureCoords, BufferUsageHint.StaticDraw);

            return true;
        }
    }
}